@@ -25,16 +25,16 @@
  * SECTION:element-cmmlenc
  * @see_also: cmmldec, oggmux
  *
- * <refsect2>
- * <para> Cmmlenc encodes a CMML document into a CMML stream.  <ulink
+ * Cmmlenc encodes a CMML document into a CMML stream.  <ulink
  * url="http://www.annodex.net/TR/draft-pfeiffer-cmml-02.html">CMML</ulink> is
  * an XML markup language for time-continuous data maintained by the <ulink
  * url="http:/www.annodex.org/">Annodex Foundation</ulink>.
- * </para>
+ *
+ * <refsect2>
  * <title>Example pipeline</title>
- * <programlisting>
+ * |[
  * gst-launch -v filesrc location=annotations.cmml ! cmmlenc ! oggmux name=mux ! filesink location=annotated.ogg
- * </programlisting>
+ * ]|
  * </refsect2>
  */
 
@@ -259,6 +259,7 @@ gst_cmml_enc_change_state (GstElement * element, GstStateChange transition)
       gst_cmml_track_list_destroy (enc->tracks);
       enc->tracks = NULL;
       g_free (enc->preamble);
+      enc->preamble = NULL;
       gst_cmml_parser_free (enc->parser);
       break;
     }
@@ -468,7 +469,7 @@ gst_cmml_enc_parse_tag_head (GstCmmlEnc * enc, GstCmmlTagHead * head)
 flow_unexpected:
   GST_ELEMENT_ERROR (enc, STREAM, ENCODE,
       (NULL), ("got head tag before preamble"));
-  enc->flow_return = GST_FLOW_UNEXPECTED;
+  enc->flow_return = GST_FLOW_ERROR;
   return;
 push_error:
   gst_caps_unref (caps);
@@ -507,6 +508,14 @@ gst_cmml_enc_parse_tag_clip (GstCmmlEnc * enc, GstCmmlTagClip * clip)
       (gchar *) clip->track);
   if (prev_clip) {
     prev_clip_time = prev_clip->start_time;
+    if (prev_clip_time > clip->start_time) {
+      GST_ELEMENT_ERROR (enc, STREAM, ENCODE,
+          (NULL), ("previous clip start time > current clip (%s) start time",
+              clip->id));
+      enc->flow_return = GST_FLOW_ERROR;
+      return;
+    }
+
     /* we don't need the prev clip anymore */
     gst_cmml_track_list_del_clip (enc->tracks, prev_clip);
   }
@@ -526,12 +535,6 @@ gst_cmml_enc_push_clip (GstCmmlEnc * enc, GstCmmlTagClip * clip,
   gchar *clip_string;
   gint64 granulepos;
 
-  if (prev_clip_time != GST_CLOCK_TIME_NONE &&
-      prev_clip_time > clip->start_time) {
-    GST_WARNING_OBJECT (enc,
-        "previous clip start time > current clip (%s) start time", clip->id);
-  }
-
   /* encode the clip */
   clip_string =
       (gchar *) gst_cmml_parser_tag_clip_to_string (enc->parser, clip);
@@ -591,8 +594,7 @@ gst_cmml_enc_push (GstCmmlEnc * enc, GstBuffer * buffer)
 
   res = gst_pad_push (enc->srcpad, buffer);
   if (GST_FLOW_IS_FATAL (res))
-    GST_ELEMENT_ERROR (enc, STREAM, ENCODE,
-        (NULL), ("could not push buffer: %s", gst_flow_get_name (res)));
+    GST_WARNING_OBJECT (enc, "push returned: %s", gst_flow_get_name (res));
 
   return res;
 }
