@@ -16,6 +16,11 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+/**
+ * SECTION:element-mulawdec
+ *
+ * This element decodes mulaw audio. Mulaw coding is also known as G.711.
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -24,7 +29,8 @@
 #include "mulaw-decode.h"
 #include "mulaw-conversion.h"
 
-extern GstPadTemplate *mulawdec_src_template, *mulawdec_sink_template;
+extern GstStaticPadTemplate mulaw_dec_src_factory;
+extern GstStaticPadTemplate mulaw_dec_sink_factory;
 
 /* Stereo signals and args */
 enum
@@ -55,6 +61,7 @@ mulawdec_sink_setcaps (GstPad * pad, GstCaps * caps)
   GstStructure *structure;
   int rate, channels;
   gboolean ret;
+  GstCaps *outcaps;
 
   mulawdec = GST_MULAWDEC (GST_PAD_PARENT (pad));
 
@@ -64,19 +71,21 @@ mulawdec_sink_setcaps (GstPad * pad, GstCaps * caps)
   if (!ret)
     return FALSE;
 
-  if (mulawdec->srccaps)
-    gst_caps_unref (mulawdec->srccaps);
-  mulawdec->srccaps = gst_caps_new_simple ("audio/x-raw-int",
+  outcaps = gst_caps_new_simple ("audio/x-raw-int",
       "width", G_TYPE_INT, 16,
       "depth", G_TYPE_INT, 16,
       "endianness", G_TYPE_INT, G_BYTE_ORDER,
       "signed", G_TYPE_BOOLEAN, TRUE,
       "rate", G_TYPE_INT, rate, "channels", G_TYPE_INT, channels, NULL);
+  ret = gst_pad_set_caps (mulawdec->srcpad, outcaps);
+  gst_caps_unref (outcaps);
 
-  mulawdec->rate = rate;
-  mulawdec->channels = channels;
-
-  return TRUE;
+  if (ret) {
+    GST_DEBUG_OBJECT (mulawdec, "rate=%d, channels=%d", rate, channels);
+    mulawdec->rate = rate;
+    mulawdec->channels = channels;
+  }
+  return ret;
 }
 
 GType
@@ -114,8 +123,10 @@ gst_mulawdec_base_init (GstMuLawDecClass * klass)
       "Convert 8bit mu law to 16bit PCM",
       "Zaheer Abbas Merali <zaheerabbas at merali dot org>");
 
-  gst_element_class_add_pad_template (element_class, mulawdec_src_template);
-  gst_element_class_add_pad_template (element_class, mulawdec_sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&mulaw_dec_src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&mulaw_dec_sink_factory));
   gst_element_class_set_details (element_class, &mulawdec_details);
 }
 
@@ -133,12 +144,13 @@ static void
 gst_mulawdec_init (GstMuLawDec * mulawdec)
 {
   mulawdec->sinkpad =
-      gst_pad_new_from_template (mulawdec_sink_template, "sink");
+      gst_pad_new_from_static_template (&mulaw_dec_sink_factory, "sink");
   gst_pad_set_setcaps_function (mulawdec->sinkpad, mulawdec_sink_setcaps);
   gst_pad_set_chain_function (mulawdec->sinkpad, gst_mulawdec_chain);
   gst_element_add_pad (GST_ELEMENT (mulawdec), mulawdec->sinkpad);
 
-  mulawdec->srcpad = gst_pad_new_from_template (mulawdec_src_template, "src");
+  mulawdec->srcpad =
+      gst_pad_new_from_static_template (&mulaw_dec_src_factory, "src");
   gst_pad_use_fixed_caps (mulawdec->srcpad);
   gst_element_add_pad (GST_ELEMENT (mulawdec), mulawdec->srcpad);
 }
@@ -153,12 +165,21 @@ gst_mulawdec_chain (GstPad * pad, GstBuffer * buffer)
   GstBuffer *outbuf;
   GstFlowReturn ret;
 
-  mulawdec = GST_MULAWDEC (gst_pad_get_parent (pad));
+  mulawdec = GST_MULAWDEC (GST_PAD_PARENT (pad));
+
+  if (G_UNLIKELY (mulawdec->rate == 0))
+    goto not_negotiated;
 
   mulaw_data = (guint8 *) GST_BUFFER_DATA (buffer);
   mulaw_size = GST_BUFFER_SIZE (buffer);
 
-  outbuf = gst_buffer_new_and_alloc (mulaw_size * 2);
+  ret =
+      gst_pad_alloc_buffer_and_set_caps (mulawdec->srcpad,
+      GST_BUFFER_OFFSET_NONE, mulaw_size * 2, GST_PAD_CAPS (mulawdec->srcpad),
+      &outbuf);
+  if (ret != GST_FLOW_OK)
+    goto alloc_failed;
+
   linear_data = (gint16 *) GST_BUFFER_DATA (outbuf);
 
   /* copy discont flag */
@@ -171,7 +192,7 @@ gst_mulawdec_chain (GstPad * pad, GstBuffer * buffer)
         mulaw_size * 2, 2 * mulawdec->rate * mulawdec->channels);
   else
     GST_BUFFER_DURATION (outbuf) = GST_BUFFER_DURATION (buffer);
-  gst_buffer_set_caps (outbuf, mulawdec->srccaps);
+  gst_buffer_set_caps (outbuf, GST_PAD_CAPS (mulawdec->srcpad));
 
   mulaw_decode (mulaw_data, linear_data, mulaw_size);
 
@@ -179,9 +200,21 @@ gst_mulawdec_chain (GstPad * pad, GstBuffer * buffer)
 
   ret = gst_pad_push (mulawdec->srcpad, outbuf);
 
-  gst_object_unref (mulawdec);
-
   return ret;
+
+  /* ERRORS */
+not_negotiated:
+  {
+    GST_ERROR_OBJECT (mulawdec, "no format negotiated");
+    gst_buffer_unref (buffer);
+    return GST_FLOW_NOT_NEGOTIATED;
+  }
+alloc_failed:
+  {
+    GST_ERROR_OBJECT (mulawdec, "pad alloc failed");
+    gst_buffer_unref (buffer);
+    return ret;
+  }
 }
 
 static GstStateChangeReturn
@@ -201,10 +234,8 @@ gst_mulawdec_change_state (GstElement * element, GstStateChange transition)
 
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
-      if (dec->srccaps) {
-        gst_caps_unref (dec->srccaps);
-        dec->srccaps = NULL;
-      }
+      dec->rate = 0;
+      dec->channels = 0;
       break;
     default:
       break;
