@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2005> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2005> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -28,10 +28,11 @@
 
 /* elementfactory information */
 static const GstElementDetails gst_rtp_mp2tdepay_details =
-GST_ELEMENT_DETAILS ("RTP packet depayloader",
+GST_ELEMENT_DETAILS ("RTP MPEG Transport Stream depayloader",
     "Codec/Depayloader/Network",
     "Extracts MPEG2 TS from RTP packets (RFC 2250)",
-    "Wim Taymans <wim@fluendo.com>");
+    "Wim Taymans <wim.taymans@gmail.com>\n"
+    "Thijs Vermeir <thijs.vermeir@barco.com>");
 
 /* RtpMP2TDepay signals and args */
 enum
@@ -40,10 +41,12 @@ enum
   LAST_SIGNAL
 };
 
+#define DEFAULT_SKIP_FIRST_BYTES	0
+
 enum
 {
-  ARG_0,
-  ARG_FREQUENCY
+  PROP_0,
+  PROP_SKIP_FIRST_BYTES
 };
 
 static GstStaticPadTemplate gst_rtp_mp2t_depay_src_template =
@@ -55,18 +58,22 @@ GST_STATIC_PAD_TEMPLATE ("src",
     );
 
 static GstStaticPadTemplate gst_rtp_mp2t_depay_sink_template =
-GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"video\", "
-        "clock-rate = (int) [1, MAX ], " "encoding-name = (string) \"MP2T-ES\""
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
+        "clock-rate = (int) [1, MAX ], " "encoding-name = (string) \"MP2T-ES\";"
         /* All optional parameters
          *
          * "profile-level-id=[1,MAX]"
          * "config=" 
          */
-    )
+        "application/x-rtp, "
+        "media = (string) \"video\", "
+        "payload = (int) " GST_RTP_PAYLOAD_MP2T_STRING ", "
+        "clock-rate = (int) [1, MAX ]")
     );
 
 GST_BOILERPLATE (GstRtpMP2TDepay, gst_rtp_mp2t_depay, GstBaseRTPDepayload,
@@ -119,6 +126,12 @@ gst_rtp_mp2t_depay_class_init (GstRtpMP2TDepayClass * klass)
   gobject_class->set_property = gst_rtp_mp2t_depay_set_property;
   gobject_class->get_property = gst_rtp_mp2t_depay_get_property;
 
+  g_object_class_install_property (gobject_class, PROP_SKIP_FIRST_BYTES,
+      g_param_spec_uint ("skip-first-bytes",
+          "Skip first bytes",
+          "The amount of bytes that need to be skipped at the beginning of the payload",
+          0, G_MAXUINT, 0, G_PARAM_READWRITE));
+
   gstelement_class->change_state = gst_rtp_mp2t_depay_change_state;
 }
 
@@ -126,32 +139,32 @@ static void
 gst_rtp_mp2t_depay_init (GstRtpMP2TDepay * rtpmp2tdepay,
     GstRtpMP2TDepayClass * klass)
 {
+  rtpmp2tdepay->skip_first_bytes = DEFAULT_SKIP_FIRST_BYTES;
 }
 
 static gboolean
 gst_rtp_mp2t_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
 {
-
+  GstCaps *srccaps;
   GstStructure *structure;
   GstRtpMP2TDepay *rtpmp2tdepay;
-  GstCaps *srccaps;
-  gint clock_rate = 90000;      /* default */
+  gint clock_rate;
+  gboolean res;
 
   rtpmp2tdepay = GST_RTP_MP2T_DEPAY (depayload);
 
   structure = gst_caps_get_structure (caps, 0);
-
-  if (gst_structure_has_field (structure, "clock-rate")) {
-    gst_structure_get_int (structure, "clock-rate", &clock_rate);
-  }
-
+  if (!gst_structure_get_int (structure, "clock-rate", &clock_rate))
+    clock_rate = 90000;         /* default */
   depayload->clock_rate = clock_rate;
 
   srccaps = gst_caps_new_simple ("video/mpegts",
-      "systemstream", G_TYPE_BOOLEAN, TRUE,
-      "packetsize", G_TYPE_INT, 188, NULL);
+      "packetsize", G_TYPE_INT, 188,
+      "systemstream", G_TYPE_BOOLEAN, TRUE, NULL);
+  res = gst_pad_set_caps (GST_BASE_RTP_DEPAYLOAD_SRCPAD (depayload), srccaps);
+  gst_caps_unref (srccaps);
 
-  return TRUE;
+  return res;
 }
 
 static GstBuffer *
@@ -160,35 +173,28 @@ gst_rtp_mp2t_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
   GstRtpMP2TDepay *rtpmp2tdepay;
   GstBuffer *outbuf;
   gint payload_len;
-  guint8 *payload;
-  guint32 timestamp;
 
   rtpmp2tdepay = GST_RTP_MP2T_DEPAY (depayload);
 
-  if (!gst_rtp_buffer_validate (buf))
-    goto bad_packet;
-
   payload_len = gst_rtp_buffer_get_payload_len (buf);
-  payload = gst_rtp_buffer_get_payload (buf);
 
-  timestamp = gst_rtp_buffer_get_timestamp (buf);
+  if (G_UNLIKELY (payload_len <= rtpmp2tdepay->skip_first_bytes))
+    goto empty_packet;
 
-  outbuf = gst_buffer_new_and_alloc (payload_len);
-  memcpy (GST_BUFFER_DATA (outbuf), payload, payload_len);
-
-  gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
-  GST_BUFFER_TIMESTAMP (outbuf) =
-      gst_util_uint64_scale_int (timestamp, GST_SECOND, depayload->clock_rate);
+  outbuf =
+      gst_rtp_buffer_get_payload_subbuffer (buf, rtpmp2tdepay->skip_first_bytes,
+      -1);
 
   GST_DEBUG ("gst_rtp_mp2t_depay_chain: pushing buffer of size %d",
       GST_BUFFER_SIZE (outbuf));
 
   return outbuf;
 
-bad_packet:
+  /* ERRORS */
+empty_packet:
   {
     GST_ELEMENT_WARNING (rtpmp2tdepay, STREAM, DECODE,
-        ("Packet did not validate"), (NULL));
+        (NULL), ("Packet was empty"));
     return NULL;
   }
 }
@@ -202,6 +208,9 @@ gst_rtp_mp2t_depay_set_property (GObject * object, guint prop_id,
   rtpmp2tdepay = GST_RTP_MP2T_DEPAY (object);
 
   switch (prop_id) {
+    case PROP_SKIP_FIRST_BYTES:
+      rtpmp2tdepay->skip_first_bytes = g_value_get_uint (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -217,6 +226,9 @@ gst_rtp_mp2t_depay_get_property (GObject * object, guint prop_id,
   rtpmp2tdepay = GST_RTP_MP2T_DEPAY (object);
 
   switch (prop_id) {
+    case PROP_SKIP_FIRST_BYTES:
+      g_value_set_uint (value, rtpmp2tdepay->skip_first_bytes);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
