@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2006> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2006> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -32,10 +32,10 @@ GST_DEBUG_CATEGORY_STATIC (rtpmp4gpay_debug);
 
 /* elementfactory information */
 static const GstElementDetails gst_rtp_mp4gpay_details =
-GST_ELEMENT_DETAILS ("RTP packet payloader",
+GST_ELEMENT_DETAILS ("RTP MPEG4 ES payloader",
     "Codec/Payloader/Network",
     "Payload MPEG4 elementary streams as RTP packets (RFC 3640)",
-    "Wim Taymans <wim@fluendo.com>");
+    "Wim Taymans <wim.taymans@gmail.com>");
 
 static GstStaticPadTemplate gst_rtp_mp4g_pay_sink_template =
     GST_STATIC_PAD_TEMPLATE ("sink",
@@ -52,9 +52,9 @@ GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) { \"video\", \"audio\", \"application\" }, "
-        "payload = (int) [ 96, 127 ], "
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
         "clock-rate = (int) [1, MAX ], "
-        "encoding-name = (string) \"mpeg4-generic\", "
+        "encoding-name = (string) \"MPEG4-GENERIC\", "
         /* required string params */
         "streamtype = (string) { \"4\", \"5\" }, "      /* 4 = video, 5 = audio */
         /* "profile-level-id = (string) [1,MAX], " */
@@ -77,24 +77,16 @@ GST_STATIC_PAD_TEMPLATE ("src",
         /* "auxiliarydatasizelength = (string) [0, 64]" */ )
     );
 
-enum
-{
-  PROP_0,
-};
-
 
 static void gst_rtp_mp4g_pay_class_init (GstRtpMP4GPayClass * klass);
 static void gst_rtp_mp4g_pay_base_init (GstRtpMP4GPayClass * klass);
 static void gst_rtp_mp4g_pay_init (GstRtpMP4GPay * rtpmp4gpay);
 static void gst_rtp_mp4g_pay_finalize (GObject * object);
 
-static void gst_rtp_mp4g_pay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec);
-static void gst_rtp_mp4g_pay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec);
-
 static gboolean gst_rtp_mp4g_pay_setcaps (GstBaseRTPPayload * payload,
     GstCaps * caps);
+static GstStateChangeReturn gst_rtp_mp4g_pay_change_state (GstElement * element,
+    GstStateChange transition);
 static GstFlowReturn gst_rtp_mp4g_pay_handle_buffer (GstBaseRTPPayload *
     payload, GstBuffer * buffer);
 
@@ -151,11 +143,10 @@ gst_rtp_mp4g_pay_class_init (GstRtpMP4GPayClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
-  gobject_class->set_property = gst_rtp_mp4g_pay_set_property;
-  gobject_class->get_property = gst_rtp_mp4g_pay_get_property;
-
   gobject_class->finalize = gst_rtp_mp4g_pay_finalize;
 
+  gstelement_class->change_state = gst_rtp_mp4g_pay_change_state;
+
   gstbasertppayload_class->set_caps = gst_rtp_mp4g_pay_setcaps;
   gstbasertppayload_class->handle_buffer = gst_rtp_mp4g_pay_handle_buffer;
 
@@ -181,9 +172,17 @@ gst_rtp_mp4g_pay_finalize (GObject * object)
 
   g_object_unref (rtpmp4gpay->adapter);
   rtpmp4gpay->adapter = NULL;
+
   g_free (rtpmp4gpay->params);
   rtpmp4gpay->params = NULL;
 
+  if (rtpmp4gpay->config)
+    gst_buffer_unref (rtpmp4gpay->config);
+  rtpmp4gpay->config = NULL;
+
+  g_free (rtpmp4gpay->profile);
+  rtpmp4gpay->profile = NULL;
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -328,11 +327,12 @@ too_short:
   }
 }
 
-static void
+static gboolean
 gst_rtp_mp4g_pay_new_caps (GstRtpMP4GPay * rtpmp4gpay)
 {
   gchar *config;
   GValue v = { 0 };
+  gboolean res;
 
 #define MP4GCAPS						\
   "streamtype", G_TYPE_STRING, rtpmp4gpay->streamtype, 		\
@@ -350,10 +350,10 @@ gst_rtp_mp4g_pay_new_caps (GstRtpMP4GPay * rtpmp4gpay)
 
   /* hmm, silly */
   if (rtpmp4gpay->params) {
-    gst_basertppayload_set_outcaps (GST_BASE_RTP_PAYLOAD (rtpmp4gpay),
+    res = gst_basertppayload_set_outcaps (GST_BASE_RTP_PAYLOAD (rtpmp4gpay),
         "encoding-params", G_TYPE_STRING, rtpmp4gpay->params, MP4GCAPS);
   } else {
-    gst_basertppayload_set_outcaps (GST_BASE_RTP_PAYLOAD (rtpmp4gpay),
+    res = gst_basertppayload_set_outcaps (GST_BASE_RTP_PAYLOAD (rtpmp4gpay),
         MP4GCAPS);
   }
 
@@ -361,6 +361,7 @@ gst_rtp_mp4g_pay_new_caps (GstRtpMP4GPay * rtpmp4gpay)
   g_free (config);
 
 #undef MP4GCAPS
+  return res;
 }
 
 static gboolean
@@ -370,6 +371,7 @@ gst_rtp_mp4g_pay_setcaps (GstBaseRTPPayload * payload, GstCaps * caps)
   GstStructure *structure;
   const GValue *codec_data;
   gchar *media_type = NULL;
+  gboolean res;
 
   rtpmp4gpay = GST_RTP_MP4G_PAY (payload);
 
@@ -381,7 +383,6 @@ gst_rtp_mp4g_pay_setcaps (GstBaseRTPPayload * payload, GstCaps * caps)
     if (G_VALUE_TYPE (codec_data) == GST_TYPE_BUFFER) {
       GstBuffer *buffer;
       const gchar *name;
-      gboolean res;
 
       buffer = gst_value_get_buffer (codec_data);
       GST_LOG_OBJECT (rtpmp4gpay, "configuring codec_data");
@@ -411,12 +412,12 @@ gst_rtp_mp4g_pay_setcaps (GstBaseRTPPayload * payload, GstCaps * caps)
   if (media_type == NULL)
     goto config_failed;
 
-  gst_basertppayload_set_options (payload, media_type, TRUE, "mpeg4-generic",
+  gst_basertppayload_set_options (payload, media_type, TRUE, "MPEG4-GENERIC",
       rtpmp4gpay->rate);
 
-  gst_rtp_mp4g_pay_new_caps (rtpmp4gpay);
+  res = gst_rtp_mp4g_pay_new_caps (rtpmp4gpay);
 
-  return TRUE;
+  return res;
 
   /* ERRORS */
 config_failed:
@@ -450,7 +451,6 @@ gst_rtp_mp4g_pay_flush (GstRtpMP4GPay * rtpmp4gpay)
   while (avail > 0) {
     guint towrite;
     guint8 *payload;
-    guint8 *data;
     guint payload_len;
     guint packet_len;
 
@@ -508,15 +508,15 @@ gst_rtp_mp4g_pay_flush (GstRtpMP4GPay * rtpmp4gpay)
     payload[2] = (total & 0x1fe0) >> 5;
     payload[3] = (total & 0x1f) << 3;   /* we use 13 bits for the size, 3 bits index */
 
-    data = (guint8 *) gst_adapter_peek (rtpmp4gpay->adapter, payload_len);
-    memcpy (&payload[4], data, payload_len);
-
+    /* copy stuff from adapter to payload */
+    gst_adapter_copy (rtpmp4gpay->adapter, &payload[4], 0, payload_len);
     gst_adapter_flush (rtpmp4gpay->adapter, payload_len);
 
     /* marker only if the packet is complete */
     gst_rtp_buffer_set_marker (outbuf, avail <= payload_len);
 
-    GST_BUFFER_TIMESTAMP (outbuf) = rtpmp4gpay->first_ts;
+    GST_BUFFER_TIMESTAMP (outbuf) = rtpmp4gpay->first_timestamp;
+    GST_BUFFER_DURATION (outbuf) = rtpmp4gpay->first_duration;
 
     ret = gst_basertppayload_push (GST_BASE_RTP_PAYLOAD (rtpmp4gpay), outbuf);
 
@@ -540,7 +540,8 @@ gst_rtp_mp4g_pay_handle_buffer (GstBaseRTPPayload * basepayload,
 
   rtpmp4gpay = GST_RTP_MP4G_PAY (basepayload);
 
-  rtpmp4gpay->first_ts = GST_BUFFER_TIMESTAMP (buffer);
+  rtpmp4gpay->first_timestamp = GST_BUFFER_TIMESTAMP (buffer);
+  rtpmp4gpay->first_duration = GST_BUFFER_DURATION (buffer);
 
   /* we always encode and flush a full AU */
   gst_adapter_push (rtpmp4gpay->adapter, buffer);
@@ -549,34 +550,29 @@ gst_rtp_mp4g_pay_handle_buffer (GstBaseRTPPayload * basepayload,
   return ret;
 }
 
-static void
-gst_rtp_mp4g_pay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec)
+static GstStateChangeReturn
+gst_rtp_mp4g_pay_change_state (GstElement * element, GstStateChange transition)
 {
   GstRtpMP4GPay *rtpmp4gpay;
+  GstStateChangeReturn ret;
 
-  rtpmp4gpay = GST_RTP_MP4G_PAY (object);
+  rtpmp4gpay = GST_RTP_MP4G_PAY (element);
 
-  switch (prop_id) {
+  switch (transition) {
     default:
       break;
   }
-}
 
-static void
-gst_rtp_mp4g_pay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec)
-{
-  GstRtpMP4GPay *rtpmp4gpay;
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
-  rtpmp4gpay = GST_RTP_MP4G_PAY (object);
-
-  switch (prop_id) {
+  switch (transition) {
     default:
       break;
   }
+  return ret;
 }
 
+
 gboolean
 gst_rtp_mp4g_pay_plugin_init (GstPlugin * plugin)
 {
