@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2006> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2006> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -32,10 +32,10 @@ GST_DEBUG_CATEGORY_STATIC (rtptheoradepay_debug);
 
 /* elementfactory information */
 static const GstElementDetails gst_rtp_theora_depay_details =
-GST_ELEMENT_DETAILS ("RTP packet depayloader",
+GST_ELEMENT_DETAILS ("RTP Theora depayloader",
     "Codec/Depayloader/Network",
     "Extracts Theora video from RTP packets (draft-01 of RFC XXXX)",
-    "Wim Taymans <wim@fluendo.com>");
+    "Wim Taymans <wim.taymans@gmail.com>");
 
 static GstStaticPadTemplate gst_rtp_theora_depay_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
@@ -44,7 +44,9 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"video\", "
         "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
-        "clock-rate = (int) 90000, " "encoding-name = (string) \"THEORA\""
+        "clock-rate = (int) 90000, " "encoding-name = (string) \"THEORA\","
+        /* only support inline delivery */
+        "delivery-method = (string) \"inline\""
         /* All required parameters 
          *
          * "sampling = (string) { "YCbCr-4:2:0", "YCbCr-4:2:2", "YCbCr-4:4:4" } "
@@ -329,6 +331,7 @@ gst_rtp_theora_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   GstCaps *srccaps;
   const gchar *delivery_method;
   const gchar *configuration;
+  gboolean res;
 
   rtptheoradepay = GST_RTP_THEORA_DEPAY (depayload);
 
@@ -339,10 +342,11 @@ gst_rtp_theora_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   if (delivery_method == NULL)
     goto no_delivery_method;
 
-  if (g_strcasecmp (delivery_method, "inline")) {
+  if (!g_ascii_strcasecmp (delivery_method, "inline")) {
     /* configure string is in the caps */
-  } else if (g_strcasecmp (delivery_method, "in_band")) {
+  } else if (!g_ascii_strcasecmp (delivery_method, "in_band")) {
     /* headers will (also) be transmitted in the RTP packets */
+    goto unsupported_delivery_method;
   } else if (g_str_has_prefix (delivery_method, "out_band/")) {
     /* some other method of header delivery. */
     goto unsupported_delivery_method;
@@ -359,13 +363,13 @@ gst_rtp_theora_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
 
   /* set caps on pad and on header */
   srccaps = gst_caps_new_simple ("video/x-theora", NULL);
-  gst_pad_set_caps (depayload->srcpad, srccaps);
+  res = gst_pad_set_caps (depayload->srcpad, srccaps);
   gst_caps_unref (srccaps);
 
   /* Clock rate is always 90000 according to draft-barbato-avt-rtp-theora-01 */
   depayload->clock_rate = 90000;
 
-  return TRUE;
+  return res;
 
   /* ERRORS */
 unsupported_delivery_method:
@@ -441,9 +445,6 @@ gst_rtp_theora_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
 
   rtptheoradepay = GST_RTP_THEORA_DEPAY (depayload);
 
-  if (!gst_rtp_buffer_validate (buf))
-    goto bad_packet;
-
   payload_len = gst_rtp_buffer_get_payload_len (buf);
 
   GST_DEBUG_OBJECT (depayload, "got RTP packet of size %d", payload_len);
@@ -475,6 +476,9 @@ gst_rtp_theora_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
   F = (header & 0xc0) >> 6;
   packets = (header & 0xf);
 
+  GST_DEBUG_OBJECT (depayload, "ident: 0x%08x, F: %d, TDT: %d, packets: %d",
+      ident, F, TDT, packets);
+
   if (TDT == 0) {
     gboolean do_switch = FALSE;
 
@@ -497,9 +501,6 @@ gst_rtp_theora_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
   payload += 4;
   payload_len -= 4;
 
-  GST_DEBUG_OBJECT (depayload, "ident: %u, F: %d, TDT: %d, packets: %d", ident,
-      F, TDT, packets);
-
   /* fragmented packets, assemble */
   if (F != 0) {
     GstBuffer *vdata;
@@ -611,12 +612,6 @@ no_output:
     return NULL;
   }
   /* ERORRS */
-bad_packet:
-  {
-    GST_ELEMENT_WARNING (rtptheoradepay, STREAM, DECODE,
-        (NULL), ("Packet did not validate"));
-    return NULL;
-  }
 switch_failed:
   {
     GST_ELEMENT_ERROR (rtptheoradepay, STREAM, DECODE,
