@@ -48,8 +48,11 @@
 
 G_BEGIN_DECLS
 
-#include "gstrtsp.h"
-#include "rtsp.h"
+#include <gst/rtsp/gstrtspconnection.h>
+#include <gst/rtsp/gstrtspmessage.h>
+#include <gst/rtsp/gstrtspurl.h>
+
+#include "gstrtspext.h"
 
 #define GST_TYPE_RTSPSRC \
   (gst_rtspsrc_get_type())
@@ -67,45 +70,79 @@ G_BEGIN_DECLS
 typedef struct _GstRTSPSrc GstRTSPSrc;
 typedef struct _GstRTSPSrcClass GstRTSPSrcClass;
 
-#define GST_RTSP_LOOP_GET_COND(rtsp)     (GST_RTSPSRC_CAST(rtsp)->loop_cond)
-#define GST_RTSP_LOOP_WAIT(rtsp)         (g_cond_wait(GST_RTSP_LOOP_GET_COND (rtsp), GST_OBJECT_GET_LOCK (rtsp)))
-#define GST_RTSP_LOOP_SIGNAL(rtsp)       (g_cond_signal(GST_RTSP_LOOP_GET_COND (rtsp)))
+#define GST_RTSP_STATE_GET_LOCK(rtsp)    (GST_RTSPSRC_CAST(rtsp)->state_rec_lock)
+#define GST_RTSP_STATE_LOCK(rtsp)        (g_static_rec_mutex_lock (GST_RTSP_STATE_GET_LOCK(rtsp)))
+#define GST_RTSP_STATE_UNLOCK(rtsp)      (g_static_rec_mutex_unlock (GST_RTSP_STATE_GET_LOCK(rtsp)))
+
+#define GST_RTSP_STREAM_GET_LOCK(rtsp)   (GST_RTSPSRC_CAST(rtsp)->stream_rec_lock)
+#define GST_RTSP_STREAM_LOCK(rtsp)       (g_static_rec_mutex_lock (GST_RTSP_STREAM_GET_LOCK(rtsp)))
+#define GST_RTSP_STREAM_UNLOCK(rtsp)     (g_static_rec_mutex_unlock (GST_RTSP_STREAM_GET_LOCK(rtsp)))
+
+#define GST_RTSP_CONN_GET_LOCK(rtsp)     (GST_RTSPSRC_CAST(rtsp)->conn_rec_lock)
+#define GST_RTSP_CONN_LOCK(rtsp)         (g_static_rec_mutex_lock (GST_RTSP_CONN_GET_LOCK(rtsp)))
+#define GST_RTSP_CONN_UNLOCK(rtsp)       (g_static_rec_mutex_unlock (GST_RTSP_CONN_GET_LOCK(rtsp)))
 
 typedef struct _GstRTSPStream GstRTSPStream;
 
-#include "rtspext.h"
-
 struct _GstRTSPStream {
   gint          id;
 
-  GstRTSPSrc   *parent;
+  GstRTSPSrc   *parent; /* parent, no extra ref to parent is taken */
 
   /* pad we expose or NULL when it does not have an actual pad */
   GstPad       *srcpad;
   GstFlowReturn last_ret;
+  gboolean      added;
+  gboolean      disabled;
+  gboolean      eos;
+  gboolean      discont;
 
   /* for interleaved mode */
-  gint          channel[2];
+  guint8        channel[2];
   GstCaps      *caps;
   GstPad       *channelpad[2];
 
   /* our udp sources */
   GstElement   *udpsrc[2];
+  GstPad       *blockedpad;
 
-  /* our udp sink back to the server */
-  GstElement   *udpsink;
+  /* our udp sinks back to the server */
+  GstElement   *udpsink[2];
+  GstPad       *rtcppad;
 
-  /* the session manager */
-  GstElement   *sess;
+  /* fakesrc for sending dummy data */
+  GstElement   *fakesrc;
 
   /* state */
   gint          pt;
   gboolean      container;
+  /* original control url */
+  gchar        *control_url;
+  /* fully qualified control url */
   gchar        *setup_url;
   guint32       ssrc; 
   guint32       seqbase;
+  guint64       timebase;
+
+  /* bandwidth */
+  guint         as_bandwidth;
+  guint         rs_bandwidth;
+  guint         rr_bandwidth;
 };
 
+/**
+ * GstRTSPNatMethod:
+ * @GST_RTSP_NAT_NONE: none
+ * @GST_RTSP_NAT_DUMMY: send dummy packets
+ *
+ * Different methods for trying to traverse firewalls.
+ */
+typedef enum
+{
+  GST_RTSP_NAT_NONE,
+  GST_RTSP_NAT_DUMMY
+} GstRTSPNatMethod;
+
 struct _GstRTSPSrc {
   GstBin           parent;
 
@@ -115,36 +152,62 @@ struct _GstRTSPSrc {
   GStaticRecMutex *stream_rec_lock;
   GstSegment       segment;
   gboolean         running;
+  gboolean         need_range;
   gint             free_channel;
+  GstEvent        *close_segment;
+  GstEvent        *start_segment;
+  GstClockTime     base_time;
 
-  /* cond to signal loop */
-  GCond           *loop_cond;
+  /* UDP mode loop */
   gint             loop_cmd;
+  gboolean         ignore_timeout;
+
+  /* mutex for protecting state changes */
+  GStaticRecMutex *state_rec_lock;
+
+  /* mutex for protecting the connection */
+  GStaticRecMutex *conn_rec_lock;
 
   gint             numstreams;
   GList           *streams;
   GstStructure    *props;
+  gboolean         need_activate;
 
   /* properties */
-  gchar           *location;
-  RTSPUrl         *url;
-  RTSPLowerTrans   protocols;
-  gboolean         debug;
-  guint   	   retry;
-  guint64          timeout;
+  gchar            *location;
+  gchar            *req_location; /* Sanitised URL to use in network requests */
+  GstRTSPUrl       *url;
+  GstRTSPLowerTrans protocols;
+  gboolean          debug;
+  guint   	    retry;
+  guint64           udp_timeout;
+  GTimeVal          tcp_timeout;
+  GTimeVal         *ptcp_timeout;
+  guint             latency;
+  guint             connection_speed;
+  GstRTSPNatMethod  nat_method;
 
   /* state */
-  gchar           *content_base;
-  RTSPLowerTrans   cur_protocols;
+  GstRTSPState       state;
+  gchar             *content_base;
+  GstRTSPLowerTrans  cur_protocols;
+  gboolean           tried_url_auth;
+  gchar             *addr;
+  gboolean           need_redirect;
 
   /* supported methods */
-  gint             methods;
+  gint               methods;
 
-  RTSPConnection  *connection;
-  RTSPMessage     *request;
-  RTSPMessage     *response;
+  /* session management */
+  GstElement      *session;
+  gulong           session_sig_id;
+  gulong           session_ptmap_id;
 
-  RTSPExtensionCtx *extension;
+  GstRTSPConnection  *connection;
+  gboolean            connected;
+
+  /* a list of RTSP extensions as GstElement */
+  GstRTSPExtensionList  *extensions;
 };
 
 struct _GstRTSPSrcClass {
@@ -153,10 +216,6 @@ struct _GstRTSPSrcClass {
 
 GType gst_rtspsrc_get_type(void);
 
-gboolean   gst_rtspsrc_send 		(GstRTSPSrc * src, RTSPMessage * request,
-		   			 RTSPMessage * response, RTSPStatusCode * code);
-
-
 G_END_DECLS
 
 #endif /* __GST_RTSPSRC_H__ */
