@@ -22,21 +22,16 @@
 
 /**
  * SECTION:element-osssrc
- * @short_description: record sound from your sound card using OSS
+ *
+ * This element lets you record sound using the Open Sound System (OSS).
  *
  * <refsect2>
- * <para>
- * This element lets you record sound using the Open Sound System (OSS).
- * </para>
  * <title>Example pipelines</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch -v osssrc ! audioconvert ! vorbisenc ! oggmux ! filesink location=mymusic.ogg
- * </programlisting>
- * will record sound from your sound card using OSS and encode it to an
+ * ]| will record sound from your sound card using OSS and encode it to an
  * Ogg/Vorbis file (this will only work if your mixer settings are right
  * and the right inputs enabled etc.)
- * </para>
  * </refsect2>
  */
 
@@ -147,6 +142,7 @@ gst_oss_src_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&osssrc_src_factory));
 }
+
 static void
 gst_oss_src_class_init (GstOssSrcClass * klass)
 {
@@ -243,6 +239,7 @@ gst_oss_src_init (GstOssSrc * osssrc, GstOssSrcClass * g_class)
   osssrc->fd = -1;
   osssrc->device = g_strdup (device);
   osssrc->device_name = g_strdup (DEFAULT_DEVICE_NAME);
+  osssrc->probed_caps = NULL;
 }
 
 static void
@@ -263,12 +260,23 @@ gst_oss_src_getcaps (GstBaseSrc * bsrc)
   osssrc = GST_OSS_SRC (bsrc);
 
   if (osssrc->fd == -1) {
-    caps = gst_caps_copy (gst_pad_get_pad_template_caps (GST_BASE_SRC_PAD
-            (bsrc)));
-  } else {
-    caps = gst_oss_helper_probe_caps (osssrc->fd);
+    GST_DEBUG_OBJECT (osssrc, "device not open, using template caps");
+    return NULL;                /* base class will get template caps for us */
   }
 
+  if (osssrc->probed_caps) {
+    GST_LOG_OBJECT (osssrc, "Returning cached caps");
+    return gst_caps_ref (osssrc->probed_caps);
+  }
+
+  caps = gst_oss_helper_probe_caps (osssrc->fd);
+
+  if (caps) {
+    osssrc->probed_caps = gst_caps_ref (caps);
+  }
+
+  GST_INFO_OBJECT (osssrc, "returning caps %" GST_PTR_FORMAT, caps);
+
   return caps;
 }
 
@@ -366,7 +374,7 @@ gst_oss_src_open (GstAudioSrc * asrc)
 no_permission:
   {
     GST_ELEMENT_ERROR (oss, RESOURCE, OPEN_READ,
-        (_("Could not open audio device for recording."
+        (_("Could not open audio device for recording. "
                 "You don't have permission to open the device.")),
         GST_ERROR_SYSTEM);
     return FALSE;
@@ -395,6 +403,8 @@ gst_oss_src_close (GstAudioSrc * asrc)
     oss->mixer = NULL;
   }
 
+  gst_caps_replace (&oss->probed_caps, NULL);
+
   return TRUE;
 }
 
@@ -528,12 +538,7 @@ gst_oss_src_delay (GstAudioSrc * asrc)
 static void
 gst_oss_src_reset (GstAudioSrc * asrc)
 {
-  GstOssSrc *oss;
-
-  //gint ret;
-
-  oss = GST_OSS_SRC (asrc);
-
-  /* deadlocks on my machine... */
-  //ret = ioctl (oss->fd, SNDCTL_DSP_RESET, 0);
+  /* There's nothing we can do here really: OSS can't handle access to the
+   * same device/fd from multiple threads and might deadlock or blow up in
+   * other ways if we try an ioctl SNDCTL_DSP_RESET or similar */
 }
